//
//  SFDeliciousAPi.h
//  SFDeliciousApi
//
//  Created by Matteo Rattotti on 1/9/09.
//  Copyright 2009 www.shinyfrog.net. All rights reserved.
//

#import "SFDeliciousGlobalHeader.h"
#import "SFDeliciousRequestType.h"
#import "SFDeliciousAccountType.h"
#import "SFDeliciousHTTPURLConnection.h"

#import "SFDeliciousXMLParserDelegate.h"
#import "SFDeliciousAPIDelegate.h"
#import "SFOAMutableURLRequest.h"
#import "SFOAToken.h"

#import "OAuthConsumer/OAConsumer.h"
#import "OAuthConsumer/OAPlaintextSignatureProvider.h"

@interface SFDeliciousAPI : NSObject <SFDeliciousXMLParserDelegate>{
    NSObject <SFDeliciousAPIDelegate> *_delegate;
    SFDeliciousAccountType accountType;
    NSString *_username;
    NSString *_password;
    NSMutableDictionary *_connections;
    NSString *_clientName;
    NSString *_clientVersion;
    NSString *_clientURL;
    NSString *_clientSourceToken;
	NSString *_APIDomain;
    NSString *_YahooApiDomain;
    NSString *_FeedDomain;
    
    NSOperationQueue* parsingQueue;
    SFOAToken *_authToken;
    
    
    NSMutableArray* _allBookmarks;
}

#pragma mark -
#pragma mark Constructors

+ (SFDeliciousAPI *)initWithDelegate:(NSObject *)newDelegate;
+ (SFDeliciousAPI *)initWithDelegate:(NSObject *)newDelegate accountType:(SFDeliciousAccountType) _accountType;

- (SFDeliciousAPI *)initWithDelegate:(NSObject *)newDelegate accountType:(SFDeliciousAccountType) _accountType;

#pragma mark -
#pragma mark Accessor

- (void)setUsername:(NSString *)newUsername password:(NSString *)newPassword;
- (void)setUsername:(NSString *)newUsername;
- (void)setUsername:(NSString *)newUsername authToken: (SFOAToken *) authToken;

- (NSString *)username;
- (NSString *)password;

@property (nonatomic, retain)  SFOAToken *_authToken;

#pragma mark -
#pragma mark OAuth methods

- (OAConsumer *) delibarConsumer;

#pragma mark -
#pragma mark Delicious API methods

- (NSString *) getAllBookmarks;
- (NSString *) getAllBookmarksWithOffset:(NSNumber*)offset;
- (NSString *) getBookmarksFor: (NSArray *) tags date:(NSDate *)date url:(NSString *)url hashes: (NSArray *)hashes;
- (NSString *) getRecentBookmarks;
- (NSString *) getRecentBookmarksFor: (NSString *) tag count: (int) number;
- (NSString *) getNetworkBookmarks;

- (NSString *) getAllHashes;

- (NSString *) getAllTags;
- (NSString *) getAllBundle;
- (NSString *) getLastUpdate;
- (NSString *) getSecretRSSKey;
- (NSString *) getSecretNetworkBookmarks: (NSString *) secretKey;

- (NSString *) getTagsSuggestionForUrl: (NSString *)url;
- (NSString *) postBookmark: (NSString *) url description: (NSString *) description 
                   extended: (NSString *) extended tags: (NSArray *) tags
                   date: (NSDate *)date replace: (BOOL) replace shared:(BOOL) shared;

- (NSString *) deleteBookmark: (NSString *) url;

- (NSString *) getBookmarkByUrl: (NSString *) url;

#pragma mark -
#pragma mark OAuth authentication

- (NSString *) getRequestTokenWithCallbackUrl: (NSString *) callbackUrl;
- (NSString *) getAuthorizationToken: (SFOAToken *)requestToken withVerifier: (NSString *) verifier;
- (void) refreshAuthorizationToken: (SFOAToken *) authToken;

#pragma mark -
#pragma mark SFDeliciousXMLParserDelegate methods

- (void)parsingSucceededForRequest:(NSString *)identifier 
                    ofResponseType:(SFDeliciousResponseType)responseType 
                 withParsedObjects:(NSArray *)parsedObjects;


- (void)parsingFailedForRequest:(NSString *)requestIdentifier 
                 ofResponseType:(SFDeliciousResponseType)responseType 
                      withError:(NSError *)error;
                      
@end

#pragma mark -
#pragma mark private api

@interface SFDeliciousAPI  (PrivateMethods)

- (NSString *)_encodeString:(NSString *)string;
- (NSString *)_queryStringWithBase:(NSString *)base parameters:(NSDictionary *)params prefixed:(BOOL)prefixed;
- (NSString *)_sendRequestWithMethod:(NSString *)method 
                                path:(NSString *)path 
                     queryParameters:(NSDictionary *)params
                                body:(NSString *)body 
                         requestType:(SFDeliciousRequestType)requestType 
                        responseType:(SFDeliciousResponseType)responseType;

- (NSString *)_sendDeliciousRequestWithMethod:(NSString *)method 
                                path:(NSString *)path 
                     queryParameters:(NSDictionary *)params
                                body:(NSString *)body 
                         requestType:(SFDeliciousRequestType)requestType 
                        responseType:(SFDeliciousResponseType)responseType;

- (NSString *)_sendOAuthRequestWithMethod:(NSString *)method
                                     path:(NSString *)path
                          queryParameters:(NSDictionary *)params
                                    token:(SFOAToken *)token
                              requestType:(SFDeliciousRequestType)requestType 
                             responseType:(SFDeliciousResponseType)responseType;

- (NSString *)_sendOAuthRequestWithRequest: (SFOAMutableURLRequest *)theRequest
                               requestType:(SFDeliciousRequestType)requestType 
                              responseType:(SFDeliciousResponseType)responseType;

- (void)_parseXMLForConnection:(SFDeliciousHTTPURLConnection *)connection;
- (BOOL) _isValidDelegateForSelector:(SEL)selector;
- (BOOL) _isFeedRequest: (SFDeliciousRequestType) request;

@end